set more off
set virtual on
clear
set mem 900m
set trace off

cd "$root"

capture log close
log using "brazempl/logs/repl-data.log", replace

* December Minimum Wage in Current USD
tempfile minwage
use "brazaux/auxil/minwage", clear
keep day salusd
gen year = year(day)
gen _mo = month(day)
keep if _mo==12
drop _mo day
order year
replace year = year - 1900
sort year
compress
keep if year>=90 & year<=98
mkmat salusd, matrix(salusd)
matname salusd 90 91 92 93 94 95 96 97 98, rows(1..9) explicit
mat list salusd


** CREATE DATA WITH PRIME AGE WORKERS for each year **

capture program drop replakmt
program define replakmt
  args yr
  foreach sector in "Agriculture" "Commerce" "Construction" "Manufacturing" "Services" "Other" {
  disp 
  disp in yellow "`sector' 19`yr'"
  use "rais-sp/rais/19`yr'/`sector'`yr'.dta", clear
  disp

  * Keep any FIRMID (same as May-04 data)
  * drop if firmid < 100
  
  * Keep only youth and prime age workers (same as May-04 data)
  * Keep any PIS number, including zero PIS (different from May-04 data: drop if pis<10000000000; but see treatments below)
  drop if age>6

  * Keep only employed workers on Dec 31st and not laid off on Dec 31st (same as May-04 data)
  keep if inddecemp==1
  keep if causedis==0 | causedis==.  /* drop workers who are de facto laid off on Dec 31st */
  drop inddecemp mofire causedis
 
  * Reset unknown CBOs to .90 codes, match ISCO occupation codes
  sort occupcbo
  merge occupcbo using "rais/cbo/cbo2isco", nokeep
  label values occupcbo cbotrans
  tab _m
  qui count if _m~=3 & occupcbo~=99910 & occupcbo~=99920 & occupcbo~=99930
  if `r(N)' > 0 {
    disp in red "Undefined CBO occupations for " in yellow "`r(N)'" in red " workers reset to .90"
    tab occupcbo if _m~=3 & occupcbo~=99910 & occupcbo~=99920 & occupcbo~=99930, nolab
    replace occupcbo = 99930 if occupcbo>99930
    replace occupcbo = int(occupcbo/100)*100 + 90 if _m~=3 & occupcbo<99900
    drop _m
    sort occupcbo
    merge occupcbo using "rais/cbo/cbo2isco", nokeep update
    tab _m
    tab occupcbo if _m~=3 & occupcbo~=99910 & occupcbo~=99920 & occupcbo~=99930, nolab
    qui count if _m~=3 & occupcbo~=99910 & occupcbo~=99920 & occupcbo~=99930
    }
  qui count if _m~=3
  replace occupcbo = 99930 if _m~=3 & occupcbo~=99910 & occupcbo~=99920
  drop _*
  order workerid firmid plantid mohire year typehir schoolng gender age national wage decwage hourswk occupcbo occupisc tenure typeemp typeest sector
  sort workerid firmid plantid mohire
  compress

  * Transform to USD wages. Create age, skill, tenure, potential experience, gender variables
  local minw = salusd[`yr'-89,1]
  keep worker firm plant year wage decwage tenure age occupisc school gender sector
    foreach var in wage decwage {
        replace `var' = `var' * 12 * `minw'
        rename `var' `var'yr
        }
  label var wageyr    "Annual Avg. Wage (usd)"
  label var decwageyr "Annualized Dec. Wage (usd)"
  replace tenure = tenure/52
  rename tenure tenuryr
  label var tenuryr "Tenure at firm (in years)"
  * tab age
  gen workrage = 12 if age==1
  replace workrage =   16 if age==2
  replace workrage =   21 if age==3
  replace workrage =   27 if age==4
  replace workrage = 34.5 if age==5
  replace workrage = 44.5 if age==6
  label var workrage "Mid-point Age of Worker in Age Group"
  gen potexper     = workrage -  6 if school==1
  replace potexper = workrage -  7 if school==2
  replace potexper = workrage - 10 if school==3
  replace potexper = workrage - 11 if school==4
  replace potexper = workrage - 14 if school==5
  replace potexper = workrage - 15 if school==6
  replace potexper = workrage - 18 if school==7
  replace potexper = workrage - 19 if school==8
  replace potexper = workrage - 22 if school==9
  label var potexper "Potential Labor Force Experience"
  drop workrage
  gen potxper2 = potexper*potexper
  label var potxper2 "Quadratic Experience Term"
  gen potxper3 = potxper2*potexper
  label var potxper3 "Cubic Experience Term"
  gen potxper4 = potxper3*potexper
  label var potxper4 "Quartic Experience Term"
  gen tenr2 = tenuryr*tenuryr
  label var tenr2 "Quadratic Tenure Term"
  gen tenr3 = tenr2*tenuryr
  label var tenr3 "Cubic Tenure Term"
  gen tenr4 = tenr3*tenuryr
  label var tenr4 "Quartic Tenure Term"
  gen byte indchild = (age==1)
  label var indchild "Child (10-14y.)"
  gen byte indyouth = (age==2)
  label var indyouth "Youth (15-17y.)"
  gen byte indadol = (age==3)
  label var indadol "Adolescent (18-24y.)"
  gen byte indnasc = (age==4)
  label var indnasc "Nascent Career (25-29y.)"
  gen byte indearly = (age==5)
  label var indearly "Early Career (30-39y.)"
  gen byte indpeak = (age==6)
  label var indpeak  "Peak Career (40-49y.)"
  gen byte indlate = (age==7)
  label var indlate  "Late Career (50-64y.)"
  gen byte indretir = (age==8)
  label var indretir "Post Retirement Age (65y.-)"
  gen byte indprof = (int(occupisc/1000)==1 | int(occupisc/1000)==2)
  label var indprof "Professional or Managerial Occ."
  gen byte indtech = (int(occupisc/1000)==3)
  label var indtech "Technical or Supervisory Occup."
  gen byte indwhit = (int(occupisc/1000)==4 | int(occupisc/1000)==5)
  label var indwhit "Other White Collar Occupation"
  gen byte indsklb = (int(occupisc/1000)==6 | int(occupisc/1000)==7 | int(occupisc/1000)==8)
  label var indsklb "Skilled Blue Collar Occupation"
  gen byte indunsklb = (int(occupisc/1000)==9)
  label var indunsklb "Unskilled Blue Collar Occupation"
  drop occup
  qui tab schoolng, gen(indschlg)
  forvalues schllvl = 1/9 {
    local lbl : label educ `schllvl'
    label var indschlg`schllvl' "`lbl'"
    }
  gen byte indprim = (school==1 | school==2 | school==3 | school==4 | school==5)
  label var indprim "Primary or Middle School Educ."
  gen byte indhigh = (school==6 | school==7)
  label var indhigh "High School Education"
  gen byte indprim_alt = (school==1 | school==2 | school==3)
  label var indprim_alt "Primary School Education"
  gen byte indhigh_alt = (school==4 | school==5 | school==6 | school==7)
  label var indhigh_alt "Middle or High School Education"
  gen byte indscoll = (school==8)
  label var indscoll "Some College Education"
  gen byte indgrad = (school==9)
  label var indgrad "Complete College Education"
  drop schoolng
  gen byte indfem = (gender==1)
  label var indfem "Female"
  foreach var in wage decwage {
      gen ln`var' = log(`var'yr)
      local lbl : variable label `var'yr
      local lbl = "Log " + "`lbl'"
      label var ln`var' "`lbl'"
      }
  drop gender
  compress
  sort worker firm plant

  drop indchild indyouth indadol indnasc indearly indpeak indlate indretir 
  keep workerid firmid plantid year age wageyr decwageyr lnwage lndecwage tenuryr tenr2 tenr3 tenr4 potexper potxper2 potxper3 potxper4 indschlg* indprim indhigh indprim_alt indhigh_alt indscoll indgrad indprof indtech indwhit indsklb indunsklb indfem sector
  order workerid firmid plantid year age wageyr decwageyr lnwage lndecwage tenuryr tenr2 tenr3 tenr4 potexper potxper2 potxper3 potxper4 indschlg* indprim indhigh indprim_alt indhigh_alt indscoll indgrad indprof indtech indwhit indsklb indunsklb indfem sector
  sort workerid firmid year
  compress
  tempfile `sector'`yr'
  disp in green "File " in yellow "`sector'`yr'"
  save ``sector'`yr''
 }

 * Note: Firms appear in more than one sector file, so sector data need to be appended before further treatments
 use `Agriculture`yr'', clear
 foreach appsec in "Commerce" "Construction" "Manufacturing" "Services" "Other" {
   qui append using ``appsec'`yr''
   }

  * Remove multiple employments (or contradictions) on Dec 31st if workerid non-zero
  * Keep a worker's job observation with highest average wage level; if tie, drop randomly
  * Differences to May-04 data
  *   May-04: drop multipis (equivalent to drop if _dupl>0 after duplicates tag workerid, gen(_dupl))
  *   May-04: drop if pis<10000000000
  gen byte indgoodpis = (workerid>=10000000000)
  label var indgoodpis "Indic.: Valid PIS number (11dig)"
  sort workerid
  duplicates tag workerid, generate(_jobs_pis)
  replace _jobs_pis = _jobs_pis + 1
  label var _jobs_pis "No. of jobs held by PIS on Dec 31st"
  tab _jobs_pis
  tab _jobs_pis if indgoodpis==1
  egen _maxwage = max(decwage) if _jobs_pis>1 & workerid~=0, by(workerid)
  drop if wage ~= _maxwage & _jobs_pis>1 & workerid~=0
  drop _max _jobs_pis
  duplicates drop workerid if workerid~=0, force

  capture label drop sector
  label define sector 1 "Agriculture" 2 "Commerce" 3 "Construction" 4 "Manufacturing" 5 "Services" 9 "Other"  
  * capture label drop brsetor
  * label define brsetor 1 "Agropecuaria" 2 "Comercio" 3 "Construcao" 4 "Industria" 5 "Servico" 9 "Outros"
  label values sector sector

  * finalize
  drop year
  compress
  sort worker firm plant
  save "brazempl/use-brazempl-`yr'", replace

  * skill ratio per firm
  keep firmid workerid indprof indtech indwhit
  gen byte indworkr = 1
  drop workerid
  collapse (sum) ind*, by(firmid)
  gen labf_skil_tot = (indprof + indtech)/indworkr
  gen labf_wh_tot = (indprof + indtech + indwhit)/indworkr
  gen ln_empl = ln(indworkr)
  keep firmid labf_skil_tot labf_wh_tot ln_empl
  label var labf_skil_tot "Share Engin'rs, Tech's, Managers"
  label var labf_wh_tot "Share White Collar Workers"
  label var ln_empl "Log Employment"
  compress
  sort firmid
  save "brazempl/data/firm-skills-`yr'", replace

end

replakmt 90
*YR replakmt 91
*YR replakmt 92
*YR replakmt 93
*YR replakmt 94
*YR replakmt 95
*YR replakmt 96
replakmt 97
*YR replakmt 98

log close
cd "$home"
