*! version 1.1.5  24oct2006  by Marc Muendler  muendler@ucsd.edu
* Nadaraya-Watson regression
* Based on Wolfgang Haerdle (1991, Ch. 2 and 5): Smoothing techniques. Springer, Berlin
* Indirect slope estimator based on Thomas M. Stoker (1994): Equivalence of direct, indirect, and slope estimators of average derivatives, In: Nonparametric and Semiparametric Methods in Econometrics and Statistics, Fifth International Symposium. Cambridge U, Cambrigde, pp. 99-118

program define nadarwats, rclass sortpreserve
    version 8.2

    * Generate(yname): Saves Conditional Mean as yname
    * Generate(xname yname): Saves Grid Points as xname and Conditional Mean as yname
    * Generate(xname yname zname): Saves Grid Points as xname, Conditional Mean as yname and (Unconditional) Density as zname
    * Generate(xname yname zname zd1name): as above, and first derivative of (Unconditional) Density as zd1name [for slope estimation following Stoker]
    * Generate(xname yname zname zd1name zd2name): as above, and second derivative of (Unconditional) Density as zd2name [for confidence interval estimation following Haerdle]
    
    * Permissible kernels: BIweight TRIweight COSine EPanechnikov GAUssian PARzen RECtangle TRIangle HORowitz MARtins
    * Permissible penalizing functions: shibata gencval akaike finitepe ricest

    syntax varlist(min=2 max=2) [if] [in] [, ///
        Width(real 0.0)     ///
        N(integer 50)       ///
        AT(varname)         ///
        noGRaph             ///
        Generate(string)    ///
        BIweight            ///
        TRIweight           ///
        COSine              ///
        EPanechnikov        ///
        GAUssian            ///
        PARzen              ///
        RECtangle           ///
        TRIangle            ///
        HORowitz            ///
        MARtins             ///
        PENalize(string)    ///
        NONZero             ///
        Symbol(string)      ///
        Connect(string)     ///
        Title(string)       ///
        *               ///
        ]

    if `"`graph'"' != "" {
        _get_gropts , graphopts(`options')
        syntax varlist(min=2 max=2)     ///
            [if] [in] [fw aw iw] [,     ///
            Width(real 0.0)     ///
            N(integer 50)       ///
            AT(varname)         ///
            noGRaph             ///
            Generate(string)    ///
            BIweight            ///
            TRIweight           ///
            COSine              ///
            EPanechnikov        ///
            GAUssian            ///
            PARzen              ///
            RECtangle           ///
            TRIangle            ///
            HORowitz            ///
            MARtins             ///
            PENalize(string)    ///
            NONZero             ///
            ]
        }
    _get_gropts , graphopts(`options')

    // check syntax
    if "`at'"!="" & `n'!=50 {
      di in red "may not specify both the at() and n() options"
      exit 198
      }
    local kern `biweight'  ///
            `triweight'  ///
            `cosine'    ///
            `epanechnikov'  ///
            `gaussian'  ///
            `parzen'    ///
            `rectangle' ///
            `triangle'  ///
            `horowitz'  ///
            `martins'
    local k : word count `kern'
    if `k' > 1 {
      di in red `"only one kernel may be specified"'
      exit 198
      }
    if `k' == 0 {
      local kern "epanechnikov"
      }
    local kernel = upper(substr(`"`kern'"',1,1)) + substr(`"`kern'"',2,.)

    local penpermis "shibata gencval akaike finitepe ricest"
    local chkpen : list penalize - penpermis
    local pencnt : word count `penalize'
    if `"`chkpen'"' != `""' | `pencnt'>1 {
      di in red `"penalizing function not allowed"'
      exit 198     
      } 

    local nsave 0
    tokenize `generate'
    local wc : word count `generate'
    if `wc' { 
      if `wc' == 1 {
        if `"`at'"' == `""'  error 198
        confirm new var `1'
        local yname  `"`1'"'
        local xname `"`at'"'
        local nsave 1
        }
      else if `wc' == 2 {
        confirm new var `1'
        confirm new var `2'
        local xname "`1'"
        local yname "`2'"
        local nsave 2
        }
      else if `wc' == 3 {
        confirm new var `1'
        confirm new var `2'
        confirm new var `3'
        local xname "`1'"
        local yname "`2'"
        local zname "`3'"
        local nsave 3
        }
      else if `wc' == 4 {
        if "`rectangle'" !="" | "`triangle'"!="" | "`parzen'"!="" {
          disp in red "kernel does not permit estimation of density derivatives"
          exit 198
          }
        confirm new var `1'
        confirm new var `2'
        confirm new var `3'
        confirm new var `4'
        local xname "`1'"
        local yname "`2'"
        local zname "`3'"
        local zd1name "`4'"
        local nsave 4
        }
      else if `wc' >= 5 {
        if "`rectangle'" !="" | "`triangle'"!="" | "`parzen'"!="" {
          disp in red "kernel does not permit estimation of density derivatives"
          exit 198
          }
        if `wc' != 5  error 198
        confirm new var `1'
        confirm new var `2'
        confirm new var `3'
        confirm new var `4'
        confirm new var `5'
        local xname "`1'"
        local yname "`2'"
        local zname "`3'"
        local zd1name "`4'"
        local zd2name "`5'"
        local nsave 5
        }
      }

    tokenize `varlist'
    args yvar xvar
    marksample touse
    qui count if `touse'
    if `r(N)'==0  error 2000 

    if `"`at'"' != `""' {
      qui count if `at' < . 
      local n = r(N)
      }
    else {
      if `n'==0 {
        qui count if `touse'
        local n = r(N)
        }
      if `n'==1  local n = 50
      if `n' > _N { 
        local n = _N
        noi di in gr `"(n() set to "' `n' `")"'
        }
      }

    local np `n'
    local hv `width'

    tempvar fh rh mh z kz kzy gridp gcv fd1 fd2 kzd1 kzd2
    quietly  {
      if `np' > _N { 
        preserve
        set obs `np' 
        }

      // variables
      gen double `z'  = 0 if `touse'
      if `"`at'"' != `""' {
        gen double `fh' = 0 if `touse'
        gen double `rh' = 0 if `touse'
        gen double `kz'  = 0 if `touse'
        gen double `kzy' = 0 if `touse'
        if `nsave'>=4  gen double `fd1'  = 0 if `touse'
        if `nsave'>=4  gen double `kzd1'  = 0 if `touse'
        if `nsave'==5  gen double `fd2'  = 0 if `touse'
        if `nsave'==5  gen double `kzd2'  = 0 if `touse'
        }
      else {
        gen double `fh' = 0 if _n <= `np'
        gen double `rh' = 0 if _n <= `np'
        gen double `kz'  = 0 if _n <= `np'
        gen double `kzy' = 0 if _n <= `np'
        if `nsave'>=4  gen double `fd1'  = 0 if _n <= `np'
        if `nsave'>=4  gen double `kzd1'  = 0 if _n <= `np'
        if `nsave'==5  gen double `fd2'  = 0 if _n <= `np'
        if `nsave'==5  gen double `kzd2'  = 0 if _n <= `np'
        }
      label var `fh' "Density (unconditional)"
      if `nsave'>=4  label var `fd1' "First Derivative of Density"
      if `nsave'>=5  label var `fd2' "Second Derivative of Density"

      // grid points        
      if `"`at'"' != `""' {
        summ `xvar' if `touse', meanonly
        local nuobs = r(N)
        gen `gridp' = `at' 
        tempvar obssrt
        gen `obssrt' = _n
        sort `gridp' `obssrt'
        }
      else {
        summ `xvar' if `touse', meanonly
        local nuobs = r(N)
        local maxval = r(max)
        local minval = r(min)
        local range = `maxval' - `minval'
        local inter = `range' / (`np' - 1)
        gen `gridp' = sum(`inter') + `minval' - `inter' if _n <= `np'
        }

      // rule-of-thumb bandwidth for gaussian data
      if `hv' <= 0.0 { 
        summ `yvar' if `touse', detail
        local hv = min( r(sd) , (r(p75)-r(p25))/1.349 )
        if `hv' <= 0.0 {
          local hv = r(sd)    
          }
        local hv = 0.9*`hv'/(r(N)^.20)
        }

      // pointwise density estimation
      local i = 1
      while `i' <= `np' {
        local xo = `gridp'[`i']
        replace `z' = (`xo' - `xvar') / `hv' if `touse'
        if "`biweight'"!=""      replace `kz' = (15/16)*(1-`z'*`z')^2       if abs(`z') < 1   // aka quartic
        if "`triweight'"!=""     replace `kz' = (35/32)*(1-`z'*`z')^3       if abs(`z') < 1   // not in kdensity
        if "`cosine'"!=""        replace `kz' = 1 + cos(2*_pi*`z')          if abs(`z') < .5       // as in kdensity, different from Haerdle's preferred choice
        if "`epanechnikov'"!=""  replace `kz' = (3/4)*(1-`z'*`z'/5)/sqrt(5) if abs(`z') < sqrt(5)  // as in kdensity, different from Haerdle's preferred choice
        if "`gaussian'"!=""      replace `kz' = (1/(sqrt(2*_pi)))*exp(-.5*`z'*`z')
        if "`rectangle'" !=""    replace `kz' = 1/2                         if abs(`z') < 1
        if "`triangle'"!=""      replace `kz' = 1 - abs(`z')                if abs(`z') < 1   // aka uniform
        if "`horowitz'"!=""      replace `kz' = (21/64)*(1-5*(`z'/5)*(`z'/5)+7*(`z'/5)*(`z'/5)*(`z'/5)*(`z'/5)-3*(`z'/5)*(`z'/5)*(`z'/5)*(`z'/5)*(`z'/5)*(`z'/5))  if abs(`z') < 5  // not in kdensity (Horowitz, JEmetrics 58 1993, 49-70)
        if "`martins'"!=""       replace `kz' = (1/(sqrt(2*_pi)))*exp(-`z'*`z'/2)*(3/2-`z'*`z'/2)  // not in kdensity (Martins, JApplEcon 16 2001, 23-39)
        if "`parzen'"!=""        {
                                 replace `kz' = 4/3-8*`z'*`z'+8*abs(`z')*abs(`z')*abs(`z')  if abs(`z') < .5
                                 replace `kz' = 8*(1-abs(`z')*abs(`z')*abs(`z'))/3          if abs(`z') >= .5 & abs(`z') < 1
                                 }
        if `nsave'==4 {
          if "`biweight'"!=""      replace `kzd1' = -(15/4)*`z'*(1-`z'*`z')      if abs(`z') < 1
          if "`triweight'"!=""     replace `kzd1' = -(105/16)*`z'*(1-`z'*`z')^2  if abs(`z') < 1
          if "`cosine'"!=""        replace `kzd1' = -2*_pi*sin(2*_pi*`z')        if abs(`z') < .5        // as in kdensity, different from Haerdle's preferred choice
          if "`epanechnikov'"!=""  replace `kzd1' = -(3/10)*`z'/sqrt(5)          if abs(`z') < sqrt(5)   // as in kdensity, different from Haerdle's preferred choice
          if "`gaussian'"!=""      replace `kzd1' = -`z'*(1/(sqrt(2*_pi)))*exp(-`z'*`z'/2)
          if "`horowitz'"!=""      replace `kzd1' =  (21/500000)*`z'*(5-`z'*`z')*(125-9*`z'*`z')  if abs(`z') < 5  // not in kdensity (Horowitz, JEmetrics 58 1993, 49-70)
          if "`martins'"!=""       replace `kzd1' = -(1/(2*sqrt(2*_pi)))*exp(-`z'*`z'/2)*`z'*(5-`z'*`z') // not in kdensity (Martins, JApplEcon 16 2001, 23-39)
          }
        else if `nsave'==5 {
          if "`biweight'"!=""      replace `kzd1' = -(15/4)*`z'*(1-`z'*`z')      if abs(`z') < 1
          if "`triweight'"!=""     replace `kzd1' = -(105/16)*`z'*(1-`z'*`z')^2  if abs(`z') < 1
          if "`cosine'"!=""        replace `kzd1' = -2*_pi*sin(2*_pi*`z')        if abs(`z') < .5        // as in kdensity, different from Haerdle's preferred choice
          if "`epanechnikov'"!=""  replace `kzd1' = -(3/10)*`z'/sqrt(5)          if abs(`z') < sqrt(5)   // as in kdensity, different from Haerdle's preferred choice
          if "`gaussian'"!=""      replace `kzd1' = -`z'*(1/(sqrt(2*_pi)))*exp(-`z'*`z'/2)
          if "`horowitz'"!=""      replace `kzd1' =  (21/500000)*`z'*(5-`z'*`z')*(125-9*`z'*`z')  if abs(`z') < 5  // not in kdensity (Horowitz, JEmetrics 58 1993, 49-70)
          if "`martins'"!=""       replace `kzd1' = -(1/(2*sqrt(2*_pi)))*exp(-`z'*`z'/2)*`z'*(5-`z'*`z') // not in kdensity (Martins, JApplEcon 16 2001, 23-39)
          if "`biweight'"!=""      replace `kzd2' = -(15/4)*(1-3*`z'*`z')      if abs(`z') < 1
          if "`triweight'"!=""     replace `kzd2' = -(105/16)*(1-6*`z'*`z'+5*`z'*`z'*`z'*`z')  if abs(`z') < 1
          if "`cosine'"!=""        replace `kzd2' = -4*_pi*_pi*cos(2*_pi*`z')  if abs(`z') < .5       // as in kdensity, different from Haerdle's preferred choice
          if "`epanechnikov'"!=""  replace `kzd2' = -(3/10)/sqrt(5)            if abs(`z') < sqrt(5)  // as in kdensity, different from Haerdle's preferred choice
          if "`gaussian'"!=""      replace `kzd2' = -(1-`z'*`z')*(1/(sqrt(2*_pi)))*exp(-`z'*`z'/2)
          if "`horowitz'"!=""      replace `kzd2' = -(21/100000)*(625-210*`z'*`z'+9*`z'*`z'*`z'*`z')  if abs(`z') < 5  // not in kdensity (Horowitz, JEmetrics 58 1993, 49-70)
          if "`martins'"!=""       replace `kzd2' = -(1/(2*sqrt(2*_pi)))*exp(-`z'*`z'/2)*(5-8*`z'*`z'+`z'*`z'*`z'*`z') // not in kdensity (Martins, JApplEcon 16 2001, 23-39)
          }

        replace `kzy' = `yvar'*`kz'
        summ `kz', meanonly 
        replace `fh' = (1 / (`nuobs'*`hv')) * `r(sum)' in `i'
        summ `kzy', meanonly 
        replace `rh' = (1 / (`nuobs'*`hv')) * `r(sum)' in `i'
        if `nsave'==4 {
          summ `kzd1', meanonly 
          replace `fd1' = (1 / (`nuobs'*`hv'*`hv')) * `r(sum)' in `i'         
          }
        else if `nsave'==5 {
          summ `kzd1', meanonly 
          replace `fd1' = (1 / (`nuobs'*`hv'*`hv')) * `r(sum)' in `i'         
          summ `kzd2', meanonly 
          replace `fd2' = (1 / (`nuobs'*`hv'*`hv'*`hv')) * `r(sum)' in `i'         
          }

        if `"`at'"' != `""' {
          replace `kz'  = 0 if `touse'
          replace `kzy' = 0 if `touse'
          if `nsave'>=4  replace `kzd1' = 0 if `touse'
          if `nsave'==5  replace `kzd2' = 0 if `touse'
          }
        else {
          replace `kz'  = 0 if _n <= `np'
          replace `kzy' = 0 if _n <= `np'
          if `nsave'>=4  replace `kzd1' = 0 if _n <= `np'
          if `nsave'==5  replace `kzd2' = 0 if _n <= `np'
          }
        local i = `i' + 1
        }

      // Nadarya-Watson estimate of E[`yvar'|`xvar']
      gen double `mh' = `rh'/`fh' if _n <= `np'
      if `"`nonzero'"'!=`""'  replace `mh' = 0 if `mh'<0
      label var `mh' "Conditional Mean"
      label var `gridp' "Grid Points"

      // cross validation
      if `"`penalize'"' != `""' {
        gen double `gcv' = (`yvar'-`mh')*(`yvar'-`mh')  // score for cross validation (to be penalized below)
        // kernel evaluated at zero  
        if "`biweight'"!=""      local k0 = (15/16)     // aka quartic
        if "`triweight'"!=""     local k0 = (35/32)     // not in kdensity
        if "`cosine'"!=""        local k0 = 1 + cos(0)  // as in kdensity, different from Haerdle's preferred choice
        if "`epanechnikov'"!=""  local k0 = 3/(4*sqrt(5)) // as in kdensity, different from Haerdle's preferred choice
        if "`gaussian'"!=""      local k0 = 1/(sqrt(2*_pi))
        if "`rectangle'" !=""    local k0 = 1/2
        if "`triangle'"!=""      local k0 = 1           // aka uniform
        if "`parzen'"!=""        local k0 = 4/3
        // penalized score (penalizing functions: shibata gencval akaike finitepe ricest)
        if "`penalize'" == "shibata"   replace `gcv' = `gcv' * ( 1+2*(`k0'/`fh') )              // Shibata (1981)
        if "`penalize'" == "gencval"   replace `gcv' = `gcv' / ( (1-`k0'/`fh')*(1-`k0'/`fh') )  // Craven & Waba (1979), Li (1985)
        if "`penalize'" == "akaike"    replace `gcv' = `gcv' * exp(2*(`k0'/`fh'))               // Akaike (1970)
        if "`penalize'" == "finitepe"  replace `gcv' = `gcv' * (1+`k0'/`fh') /  (1-`k0'/`fh')   // Akaike (1974)
        if "`penalize'" == "ricest"    replace `gcv' = `gcv' / (1-2*`k0'/`fh')                  // Rice (1984)
        summ `gcv', meanonly
        local ase `r(mean)'
        }

      // graph
      if `"`graph'"'==`""' {
         graph twoway                    ///
         (line `mh' `gridp',              ///
            ytitle(`"Conditional Mean"') ///
            xtitle(`"`Grid Points'"')    ///
            legend(cols(1))              ///
            `options'               ///
            )
      }

      // double save in S_# and r()
      if `"`at'"' != `""' {
        summ `fh' if `touse', meanonly
        local scale = 1/(`np'*`r(mean)')
        }
      else {
        summ `fh' in 1/`np', meanonly
        local scale = 1/(`np'*`r(mean)')
        }
      ret clear
      ret local kernel `"`kernel'"'
      ret scalar n = `np'           // (sic)
      ret scalar width = `hv'
      ret scalar scale = `scale'
      if `"`penalize'"' != `""'  ret scalar avgse = `ase'  // averaged squared error
      global S_1   `"`kernel'"'
      global S_2 = `np'
      global S_3 = `hv'
      global S_4 = `scale'
      if `"`penalize'"' != `""'  global S_5 = `ase'

      capture restore, not 

      // variable generation
      if `nsave' != 0 {
        if `"`nonzero'"'!=`""'  replace `fh' = 0 if `fh'<0
        rename `mh' `yname'
        if `nsave'>=2  rename `gridp' `xname'
        if `nsave'>=3  rename `fh' `zname'
        if `nsave'>=4  rename `fd1' `zd1name'
        if `nsave'==5  rename `fd2' `zd2name'
        }

    }  /* close quietly */

end
